"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SettingsPropertyCompletionProvider = void 0;
const jsonc_parser_1 = require("jsonc-parser");
const theme_check_common_1 = require("@shopify/theme-check-common");
const utils_1 = require("../../utils");
const BlockTypeCompletionProvider_1 = require("./BlockTypeCompletionProvider");
const vscode_languageserver_protocol_1 = require("vscode-languageserver-protocol");
const translations_1 = require("../../../translations");
/**
 * The SettingsPropertyCompletionProvider offers property completions for:
 * - `presets.[].settings.[]` objects inside `{% schema %}` tag in sections and blocks
 * - `default.settings` object inside `{% schema %}` tag in sections
 *
 * @example
 * {% schema %}
 * {
 *   "presets": [
 *     {
 *       "settings": [
 *         { "█" },
 *       ]
 *     },
 *   ],
 *   "default": {
 *     "settings": {
 *       "█"
 *     }
 *   }
 * }
 * {% endschema %}
 */
class SettingsPropertyCompletionProvider {
    constructor(getDefaultSchemaTranslations) {
        this.getDefaultSchemaTranslations = getDefaultSchemaTranslations;
    }
    async completeProperty(context, path) {
        if (context.doc.type !== theme_check_common_1.SourceCodeType.LiquidHtml)
            return [];
        // section files can have schemas with `presets` and `default`
        // block files can have schemas with `presets` only
        if (!((0, utils_1.isSectionFile)(context.doc.uri) &&
            (isPresetSettingsPath(path) || isDefaultSettingsPath(path))) &&
            !((0, utils_1.isBlockFile)(context.doc.uri) && isPresetSettingsPath(path))) {
            return [];
        }
        const { doc } = context;
        const schema = await doc.getSchema();
        if (!schema || !(0, BlockTypeCompletionProvider_1.isSectionOrBlockSchema)(schema)) {
            return [];
        }
        let parsedSchema;
        /**
         * Since we are auto-completing JSON properties, we could be in a state where the schema is invalid.
         * E.g.
         * {
         *   "█"
         * }
         *
         * In that case, we manually parse the schema ourselves with a more fault-tolerant approach.
         */
        if ((0, theme_check_common_1.isError)(schema.parsed)) {
            parsedSchema = (0, jsonc_parser_1.parse)(schema.value);
        }
        else {
            parsedSchema = schema.parsed;
        }
        if (!(parsedSchema === null || parsedSchema === void 0 ? void 0 : parsedSchema.settings) || !Array.isArray(parsedSchema.settings)) {
            return [];
        }
        const translations = await this.getDefaultSchemaTranslations(doc.textDocument.uri);
        return parsedSchema.settings
            .filter((setting) => setting.id)
            .map((setting) => {
            let docValue = '';
            if (setting.label) {
                if (setting.label.startsWith('t:')) {
                    const translation = (0, translations_1.translationValue)(setting.label.substring(2), translations);
                    if (translation) {
                        docValue = (0, translations_1.renderTranslation)(translation);
                    }
                }
                else {
                    docValue = setting.label;
                }
            }
            return {
                kind: vscode_languageserver_protocol_1.CompletionItemKind.Property,
                label: `"${setting.id}"`,
                insertText: `"${setting.id}"`,
                documentation: {
                    kind: 'markdown',
                    value: docValue,
                },
            };
        });
    }
}
exports.SettingsPropertyCompletionProvider = SettingsPropertyCompletionProvider;
function isPresetSettingsPath(path) {
    return path.length === 3 && path.at(0) === 'presets' && path.at(2) === 'settings';
}
function isDefaultSettingsPath(path) {
    return path.length === 2 && path.at(0) === 'default' && path.at(1) === 'settings';
}
//# sourceMappingURL=SettingsPropertyCompletionProvider.js.map